# -*- coding: utf-8 -*- 
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Agent zephir de surveillance des connexions (ip conntrack)
"""

from twisted.python import log
from twisted.internet import defer
from twisted.internet.utils import getProcessOutput

from zephir.monitor.agentmanager.agent import Agent
from zephir.monitor.agentmanager import status
from zephir.monitor.agentmanager.data import HTMLData, TableData
from zephir.monitor.agentmanager.util import percent

from pyctd.conntracking import DataCollector
from pyctd.server import serializeConnectionSet

# IP protocol numbers (/usr/include/netinet/in.h)
IPPROTO_ICMP = 1
IPPROTO_TCP = 6
IPPROTO_UDP = 17
IPPROTO_SCTP = 132

# Python dictinnoary to convert IP protocol to string
IPPROTO_NAMES = {
    IPPROTO_ICMP: "icmp",
    IPPROTO_TCP: "tcp",
    IPPROTO_UDP: "udp",
    IPPROTO_SCTP: "sctp",
}

class Connexions(Agent):

    def __init__(self, name,
                 **params):
        Agent.__init__(self, name, **params)
        self.table = TableData([
            ('id', 'Id_cnx', {'align':'right'}, None),
            ('proto', 'Protocole', {'align':'left'}, None),
            # ('mark', 'Marquage', {'align':'left'}, None),
            # ('user', 'Utilisateur', {'align':'left'}, None),
            ('timeout', 'Timeout', {'align':'left'}, None),
            # ('status', 'Etat', {'align':'left'}, None),
            ('src', 'Source', {'align':'right'}, None),
            ('dst', 'Destination', {'align':'right'}, None),
            ('rate_in', 'Débit Entrant', {'align':'right'}, None),
            ('rate_out', 'Débit Sortant', {'align':'right'}, None)])
        self.data = [self.table]
        self.collector = None

    def measure(self):
        if self.collector == None:
            self.collector = DataCollector('conntrack')
        self.collector.refresh()
        # FIXME : utilser pyctd.server.serializeConnectionSet ?
        # meas_data = serializeConnectionSet(self.collector.current)
        meas_data = []
        ctdata = self.collector.current.dict
        for id, conn in ctdata.items():
            conn_data = conn.conntrack
            tuple = conn_data.tuple[0]
            fields = {}
            fields['id'] = id
            fields['mark'] = conn_data.mark
            fields['user'] = ""
            fields['timeout'] = conn_data.timeout
            fields['status'] = conn_data.status
            fields['src'] = tuple.src.getIPv4().strNormal()
            fields['dst'] = tuple.dst.getIPv4().strNormal()
            # FIXME : affichage du protocole/ nom des champs (l4src, l4dst)
            if t.protonum == IPPROTO_ICMP:
                fields["l4src"] = int(t.l4src.icmp.id)
                fields["l4dst"] = int(t.l4dst.icmp.type)
            else:
                fields["l4src"] = int(ntohs(t.l4src.getValue(tuple.protonum)))
                fields["l4dst"] = int(ntohs(t.l4dst.getValue(tuple.protonum)))
            fields['proto'] = "%s/%s" % (fields["l4src"],fields["l4dst"])
            # fields['proto'] = tuple.protonum
            # FIXME : afficher en % de la b.p disponible
            # ou en % par rapport à l'ensemble des connexions ?
            fields['rate_in'] = float(conn.byterate[0])
            fields['rate_out'] = float(conn.byterate[1])
            meas_data.append(fields)
        return {'statistics': meas_data}

    def check_status(self):
        stat = status.OK()
        return stat

    def save_measure(self, measure):
        Agent.save_measure(self, measure)
        self.last_measure = measure
        
    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value['statistics']

