# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Agent zephir pour les services distants
"""

import socket
from creole.parsedico import parse_dico
#from twisted.python import log
from zephir.monitor.agentmanager.agent import Agent
from zephir.monitor.agentmanager.data import HTMLData, TableData
from zephir.monitor.agentmanager import status
from zephir.monitor.agentmanager.util import status_to_img, boolean_to_onoff
from twisted.internet.utils import getProcessOutput

def tcpcheck(mesures, label, address, port=80, timeout=3, fallback = None):
    """tentative de connexion sur un port donné d'un serveur distant"""
    tcpcheck = getProcessOutput('/usr/bin/tcpcheck',
                                args = [str(timeout), address+':'+str(port)],
                                env = {'LC_ALL': 'C'})
    tcpcheck.addCallback(process_tcp, mesures, label, fallback)
    return tcpcheck
  
def process_tcp(tcp_result, mesures, label, fallback):
    # traitement du résultat de la commande
    if 'alive' in tcp_result:
        status = "On"
    else:
        # si il y a une adresse de 'secours', on la teste aussi
        if fallback != None:
            return tcpcheck(mesures, 'Accès Distant', fallback)
        status = "Off"
    mesures.append({'status':status, 'service':label})
    return mesures

def resolve(address):
    """résolution d'adresse"""
    try:
        socket.gethostbyname(address)
    except socket.error:
        return False
    return True


class Web(Agent):

    def __init__(self, name, module,
                 **params):
        Agent.__init__(self, name, **params)
        self.module = module
        self.table = TableData([
            ('status', "État", {'align':'center'}, status_to_img), 
            ('service', "Service", {'align':'left'}, None),
            ])
        self.dns = 1
        self.data = [self.table]

    def measure(self):
        mesures = []
        # test dns distant
        conf_eole = parse_dico()
        try:
            status = tcpcheck(mesures, 'DNS distant 1', conf_eole['adresse_ip_dns_primaire'], 53)
            if conf_eole['adresse_ip_dns_secondaire'] != "":
                self.dns = 2
                status = tcpcheck(mesures, 'DNS distant 2', conf_eole['adresse_ip_dns_secondaire'], 53)
        except KeyError:
            # pas de variable dns_secondaire sur des fichiers de config non regénérés.
            pass
        # test de la résolution
        status = resolve('eole.orion.education.fr')
        mesures.append({'status':boolean_to_onoff(status), 'service':'Résolution'})
        # test accès distant
        return tcpcheck(mesures, 'Accès Distant', 'eole.orion.education.fr', fallback='www.cru.fr')
    
    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value

    def check_status(self):
        """remonte une erreur si un des services est tombé"""
        if self.last_measure is not None:
            err_dns = 0
            for service in self.last_measure.value:
                self.measure_data[service['service']]=service['status']
                # cas du premier dns : on permet l'erreur
                if service['service'].startswith('DNS'):
                    if service['status'] != 'On':
                        err_dns += 1
                        # si tous les dns sont en panne : erreur
                        if err_dns == self.dns:
                            return status.Error()
                else:
                    if service['status'] != 'On':
                        return status.Error()
            if err_dns > 0:
                return status.Warn()
        else:
            # pas de mesure connue
            return status.Unknown()
        return status.OK()

